from django.db import models


class TeamOwnedModel(models.Model):
    """
    Mixin for models whose instances are owned by a particular team.
    """
    def get_owning_team(self):
        """
        Gets the team that owns this object.

        :return:    The team.
        """
        raise NotImplementedError(TeamOwnedModel.get_owning_team.__qualname__)

    class Meta:
        abstract = True



from django.db import models
from django.conf import settings

from ..mixins import SoftDeleteModel, TeamOwnedModel
from ..apps import APP_NAME


class Membership(TeamOwnedModel, SoftDeleteModel):
    # Permission constants
    PERMISSION_READ = "R"
    PERMISSION_WRITE = "W"
    PERMISSION_ADMIN = "A"

    # The user whose membership this is
    user = models.ForeignKey(settings.AUTH_USER_MODEL,
                             on_delete=models.DO_NOTHING,
                             related_name="memberships")

    # The team to which this membership applies
    team = models.ForeignKey(f"{APP_NAME}.Team",
                             on_delete=models.DO_NOTHING,
                             related_name="memberships")

    # The permissions the member has within the team
    permissions = models.CharField(max_length=1,
                                   choices=[
                                       (PERMISSION_READ, "Read"),
                                       (PERMISSION_WRITE, "Write"),
                                       (PERMISSION_ADMIN, "Admin")
                                   ],
                                   default=PERMISSION_READ)

    class Meta(TeamOwnedModel.Meta, SoftDeleteModel.Meta):
        constraints = [
            # Ensure that each user is only an active member of each team at most once
            models.UniqueConstraint(name="one_active_membership_per_user_per_team",
                                    fields=["user", "team"],
                                    condition=SoftDeleteModel.active_Q)
        ]

    def is_admin(self) -> bool:
        """
        Whether the member is an admin for this team.
        """
        return self.permissions == self.PERMISSION_ADMIN

    def get_owning_team(self):
        return self.team

    def __str__(self) -> str:
        return f"User \"{self.user}\" in team \"{self.team}\""



# Generated by Django 2.2.6 on 2019-12-04 00:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('simple_django_teams', '0001_initial'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='team',
            name='unique_active_organisation_names',
        ),
        migrations.AddConstraint(
            model_name='team',
            constraint=models.UniqueConstraint(condition=models.Q(deletion_time__isnull=True), fields=('name',), name='unique_active_team_names'),
        ),
    ]



from setuptools import setup, find_packages


def _read(filename: str) -> str:
    """
    Reads in the content of the file.

    :param filename:    The file to read.
    :return:            The file content.
    """
    with open(filename, "r") as file:
        return file.read()


setup(
    name="simple-django-teams",
    description="Simple Django teams.",
    long_description=f"{_read('DESCRIPTION.rst')}\n"
                     f"{_read('CHANGES.rst')}",
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Programming Language :: Python :: 3.7',
    ],
    license='MIT',
    package_dir={
        '': 'teams/src'
    },
    packages=find_packages(where="teams/src"),
    version="0.0.2",
    author='Corey Sterling',
    author_email='coreytsterling@gmail.com',
    install_requires=[
        "django"
    ]
)
